// License: GNU General Public License (GPL)
// Copyright (c) 2008 NIPPON TELEGRAPH AND TELEPHONE CORPORATION

import org.w3c.dom.*;
import java.io.*;
import javax.xml.parsers.*;
import java.util.*;

public class failstate {
	static Hashtable nodeStandbyState = new Hashtable();
	static String diskcheckStatus = "diskcheck_status";
	static int diskcheckDefined = 0; 
	
	public static void setNodeStandbyState(Element configuration) {
		String[] TagNameList= {"instance_attributes", "attributes", "nvpair"};
		Element tag = null;

		nodeStandbyState.clear();   // Hashtableのクリア

		//<configuration> の子要素である<nodes>を取得
		NodeList nodesList = configuration.getElementsByTagName("nodes");
        	Element nodes = (Element)nodesList.item(0);
		//<nodes>の子要素である<node>を取得。複数個あるのでfor文で。
		NodeList nodeList = nodes.getElementsByTagName("node");
		for (int i = 0; i < nodeList.getLength(); i++) {
			tag = (Element)nodeList.item(i);
			String uname = tag.getAttribute("uname"); 
			//子要素があればgetNvpairList()へ
			if (tag.hasChildNodes()) {
				NodeList nvpairList = getNvpairList(tag, 0);

				for (int j = 0; j < nvpairList.getLength(); j++) {
					Element nvpair = (Element)nvpairList.item(j);
					nodeStandbyState.put(uname, nvpair.getAttribute("value"));	//値をHashtableへ。
				}
			} else { //子要素がなければ、"on"として扱う。
				nodeStandbyState.put(uname, "off");
			}	
		}	
		// System.out.println(nodeStandbyState.toString());
	}

	public static String getNodeStandbyState(String key) {
		return (String)nodeStandbyState.get(key);
	}

	public static NodeList getNvpairList(Element tag, int flag) {
		String[] TagNameList0 = {"instance_attributes", "attributes", "nvpair"};
		String[] TagNameList1 = {"transient_attributes", "attributes", "nvpair"};
		NodeList tagNameList = null;

		for (int j =0; j < 3 ; j ++) {
			if (flag == 0)  tagNameList = tag.getElementsByTagName(TagNameList0[j]);
			else if (flag == 1)  tagNameList = tag.getElementsByTagName(TagNameList1[j]);
			if (tagNameList.getLength() != 0 ) {
				tag = (Element)tagNameList.item(0);
			} else {
				return null;
			}
		}
		return tagNameList;
	}
		

	public static void setDiskcheckDefined(Element configuration) {
		final String DISKCHECK_RA= "Diskcheck";

		//<configuration> の子要素である<nodes>を取得
		NodeList resourcesList = configuration.getElementsByTagName("resources");
        Element resources = (Element)resourcesList.item(0);
		//<resources> の子要素である<clone>を取得
		NodeList cloneList = resources.getElementsByTagName("clone");
		//clone定義は複数ある可能性ありなので、for文で。
		for (int i = 0; i < cloneList.getLength(); i++) {
        	Element clone = (Element)cloneList.item(i);
			// primitive定義を取得
			NodeList primitiveList = clone.getElementsByTagName("primitive");
			// cloneの中のprimitive定義は一つのはずなので。
        	Element primitive = (Element)primitiveList.item(0);
			//属性を取得し、RAの種類を判別する。
			if ((primitive.getAttribute("type")).equals(DISKCHECK_RA)) { 
				diskcheckDefined = 1;
				NodeList nvpairList = getNvpairList(primitive, 0);
				for (int j = 0; j < nvpairList.getLength(); j ++) {
					Element nvpair = (Element)nvpairList.item(j);
					if ((nvpair.getAttribute("name")).equals("attr_name"))
						diskcheckStatus = nvpair.getAttribute("value");
				}
			}
		}
	}
		

    public static void main(String[] args) {
		final char esc = 27;
		final String clear = esc + "[2J", command = "/usr/sbin/cibadmin -Q";
		String uname, name, value, crmd, resource;
		Process process;
		InputStream is;
		Document doc;
		int	i, j, pingdValueMultiplier, numOfPingNode = 0;
		Element cib, tag,  nodeState, transientAttribute, attributes, nvpair;
		NodeList tagList, nodeStateList, transientAttributeList, attributesList, nvpairList;

		commandLine cl = new commandLine(args);	

		/* ha.cfの設定内容からping先ノード数の取得、および、-a で指定される値
		   (通常はdefault_ping_set)の取得。また-mオプションで指定される値も取得*/
		pingdFromHacf pfh = new pingdFromHacf();
		String pingdPingSet = pfh.getValue("-a");
		if (pfh.getValue("-m") != null) {
			Integer pingdMOperation = new Integer(pfh.getValue("-m"));
			pingdValueMultiplier = pingdMOperation.intValue();
			numOfPingNode = pingdValueMultiplier * pfh.numOfPingNode();
		}
		

		try {
			DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = dbfactory.newDocumentBuilder();
	
			do {
				/* /usr/sbin/cibadminを実行し、cib.xmlを取得する */
				process = Runtime.getRuntime().exec(command);
				is = process.getInputStream();
				doc = builder.parse(is);

				// root要素 <cib>を取得	
				cib = doc.getDocumentElement();

				// cib.xmlの中から、<status>を取得 <status>はcib.xmlに一つ
				tagList = cib.getElementsByTagName("configuration");
				tag = (Element)tagList.item(0);  
				
				setNodeStandbyState(tag);
				setDiskcheckDefined(tag);

				// cib.xmlの中から、<status>を取得 <status>はcib.xmlに一つ
				tagList = cib.getElementsByTagName("status");
				tag = (Element)tagList.item(0);  

				// node_stateのリストを取得 node_stateはnode数分存在
				nodeStateList = tag.getElementsByTagName("node_state");

				System.out.print(clear); //画面のクリア
		  		System.out.println("*********** failure status ************* ");
				Date date = new Date();
				System.out.println("Last updated: " + date);

				for (i = 0; i < nodeStateList.getLength(); i ++) {
					nodeState = (Element)nodeStateList.item(i);
					uname = nodeState.getAttribute("uname"); 
					//nodeの状態=offline->出力なし。
                	if ((nodeState.getAttribute("crmd")).equals("offline")) continue; 
					else System.out.println("Node: "+ uname);

					nvpairList = getNvpairList(nodeState, 1); 
					if (nvpairList == null) {
						System.out.println("    cannot get status now. Wait....");
						continue;
					}

					int dkflag = 0;
					for (j = 0; j< nvpairList.getLength(); j++) {
						nvpair = (Element)nvpairList.item(j);
						name = nvpair.getAttribute("name");
						value = nvpair.getAttribute("value");
						if ((getNodeStandbyState(uname).equals("on")) && 
								(diskcheckDefined == 1) && (dkflag == 0)) {
							System.out.println("   diskcheck: status unknown -- node is standby");
							dkflag = 1;
						}
					
						if (name.startsWith("fail-count")) {   // Resourceのfailcountを取得
                    		resource = name.substring(11);
                        	System.out.print("   ResourceID: " + resource + "  (fail-count: " + value +")");
                        	System.out.println("    Resource is failure !!");
						} else if (pingdPingSet != null && name.startsWith(pingdPingSet)) { //pingdの状態取得
							System.out.print("   pingd:"+ pingdPingSet+": "  + value);
							Integer xmlPingdValue = new Integer(value);	// valueを整数化
							if (xmlPingdValue.intValue() == numOfPingNode)  
								System.out.println("  LAN is working");
                        	else System.out.println("  LAN is failure !!");

                    	} else if (name.startsWith(diskcheckStatus)) {  //Diskcheckの状態取得
							if (dkflag == 0) {
							    System.out.print("   diskcheck:diskcheck_status: " + value);
								if (value.equals("normal")) System.out.println("  Disk is working");
                        		else System.out.println("  DISK is failure !!");
							}
						}
					}
				}
				Thread.sleep(cl.intervalTime * 1000);
		  	} while (cl.oneShotOption == 0);
		} catch (gnu.xml.dom.ls.DomLSException gxd) {
			System.out.println("cib.xmlが取得できません");
			System.out.println("実行権限、Heartbeatの稼働状況を確認してください ");
		} catch (org.xml.sax.SAXParseException spe) {
			System.out.println("cib.xmlが取得できません");
			System.out.println("実行権限、Heartbeatの稼働状況を確認してください ");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
